<?php
/*
 * This file is part of LinkMan - PHP Links Manager.
 *
 * (c) Copyright 2016 by Klemen Stirn. All rights reserved.
 * http://www.phpjunkyard.com
 * http://www.phpjunkyard.com/php-link-manager.php
 *
 * For the full copyright and license agreement information, please view
 * the docs/index.html file that was distributed with this source code.
 */

define('IN_SCRIPT', 1);
define('THIS_PAGE', 'VERIFY');

// Require the settings file
require '../config/settings.php';

// Load functions
require '../inc/common.inc.php';

// Check demo mode
if (defined('PJ_DEMO')) {
    die('This function has been disabled in demo.');
}

// Check if PHP output buffering is enabled
$settings['output_buffering'] = ini_get('output_buffering') ? true : false;

// Remove time limit as this check can take a long time
set_time_limit(0);

// Start output buffer
ob_start();

// Do we have an access key?
$access_key = pj_GET('key', false);

// If we have an access key, this is an automated check
if (strlen($access_key)) {
    // Verify the automated key
    if ($access_key != $settings['access_key']) {
        die('Invalid access key!');
    }

    // Set the key to true for other checks
    $access_key = true;
} // No access key, this is from admin interface
else {
    // Start session
    pj_session_start();

    // Are we logged in?
    pj_isLoggedIn(true);

    // Check security token
    pj_token_check() or die('Invalid security token!');

    // Print content type header and start <pre> tag
    header('Content-type: text/html; charset=utf-8');
    pj_fill_buffer();
    pj_flush();
    echo "<pre>";
}

// The settings file is in parent folder
$settings['db_file'] = '../config/' . $settings['db_file'];

$data_modified = false;

echo "Verification in progress, do not close this page until you see FINISHED at the bottom.\n";

// Veification options
$remove_site_down = intval(pj_GET('remove_site_down', 0));
$remove_page_down = intval(pj_GET('remove_page_down', 0));
$remove_not_found = intval(pj_GET('remove_not_found', 0));
$remove_nofollow  = intval(pj_GET('remove_nofollow', 0));
$remove_cheaters  = intval(pj_GET('remove_cheaters', 0));

// Verifiaction functions
require '../inc/verify.inc.php';

// Get links database
$data = explode('//', file_get_contents($settings['db_file']), 2);

// Convert contents into an array
$database = isset($data[1]) ? unserialize($data[1]) : array();
unset($data);

// Verify each links
foreach ($database as $id => $link) {
    pj_fill_buffer();
    echo "--------------------------------------------------\n";
    echo $link['F'] ? "FEATURED\n" : '';
    echo "{$link['L']}\n";
    pj_flush();

    // Make sure the website can be accessed
    if (crawl_website($link['L']) === false) {
        link_verify_error($id, "Cannot open website URL", $remove_site_down);
        continue;
    }

    // If reciprocal link is not required, skip further verifications
    if ($link['R'] == '') {
        echo "No reciprocal link required\nALL OK!\n";
        continue;
    } else {
        echo "{$link['R']}\n";
    }

    // Detect existing cheating links using "showlink" in query string
    if (strpos($link['R'], 'showlink=') !== false) {
        link_verify_error($id, "Cheating reciprocal link (showlink in query string)", $remove_cheaters);
        continue;
    }

    // Get reciprocal page contents
    if (($html = crawl_website($link['R'])) === false) {
        link_verify_error($id, "Cannot open page with reciprocal URL", $remove_page_down);
        continue;
    }

    // Is reciprocal link there?
    $reciprocal_found = reciprocal_link_found($html);

    if ($reciprocal_found === false) {
        link_verify_error($id, "Reciprocal link not found", $remove_not_found);
        continue;
    } elseif ($reciprocal_found === 'NOFOLLOW') {
        link_verify_error($id, "Reciprocal link blocked with nofollow tag", $remove_nofollow);
        continue;
    }

    // All seems OK
    echo "ALL OK!\n";
}

echo "--------------------------------------------------\n";

// Need to save any changes?
if ($data_modified) {
    // Update sorting
    uasort($database, 'sort_order');
    reset_sort_order($database);

    // Update database file
    if (@file_put_contents($settings['db_file'], "<?php die();//" . serialize($database), LOCK_EX) === false) {
        $result = "ERROR: Error writing to database file, please try again later.";
    } else {
        $result = "Database file updated successfully.";
    }
} else {
    $result = "No changes, nothing to update.";
}

echo "{$result}\n\nFINISHED!";

if (!$access_key) {
    echo '</pre>';
    echo '<script type="text/javascript">alert(\'FINISHED!\\n\\n' . addslashes($result) . '\');</script>';
}

ob_end_flush();


////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS
////////////////////////////////////////////////////////////////////////////////


function pj_fill_buffer()
{
    global $settings, $access_key;

    if ($settings['output_buffering'] && !$access_key) {
        echo str_repeat(' ', 4096) . "\n";
    }
} // END pj_fill_buffer()


function pj_flush()
{
    global $access_key;

    if (!$access_key) {
        ob_flush();
        flush();
    }
} // END pj_flush()
